unit umacro;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, ComCtrls,registry, menus, ShellApi;

const
  WM_SYSTRAY = WM_USER + $400;

  DEFAULT_REGBASE = HKEY_CURRENT_USER;
  COMMENT_CHAR = ';';
  STRING_SEP_CHAR = '\';
  LNGPAK_ID = 'LANGUAGE_PACK';
  SDUDFFILE = 'sdudf.dll';
  BYTES_TO_DUMP = $10000;
  MAX_PATCHSTREAM_LENGTH = 32;
  MAX_PATCHENTRIES = 16;
  MAX_PATCHPARAMS = 8;
  MAX_VERSIONS = 16;
  SKIP_VERSION_CHECK = -1;
  LOWEST_SUPPORTED_VERSION = 125;

type
  pBoolean = ^boolean;
  pByte = ^byte;

  tFileInfo = packed record
    CompanyName: string;
    FileDescription: string;
    FileVersion: string;
    InternalName: string;
    LegalCopyright: string;
    LegalTrademarks: string;
    OriginalFilename: string;
    ProductName: string;
    ProductVersion: string;
    Comments: string;
    PrivateBuild: string;
    SpecialBuild: string;
  end;

  tPatchEntryParamEntry = record
                           index:byte;
                           paramptr:pByte;
                          end;
  tPatchEntry = record
                 switch_on:pBoolean;
                 applicable:boolean;
                 version:integer;
                 num_of_patchbytes:byte;
                 num_of_params:byte;
                 src:array [0..MAX_PATCHSTREAM_LENGTH-1] of byte;
                 dst:array [0..MAX_PATCHSTREAM_LENGTH-1] of byte;
                 param:array [0..MAX_PATCHPARAMS-1] of tPatchEntryParamEntry;
                 location:longint;
                end;
  tPatch = record
             num_of_patches:byte;
             patch:array [0..MAX_PATCHENTRIES-1] of tPatchEntry;
           end;

  {
     TSystray component written by Tobias Ylander. Any corrections, questions or
     comments can be sent to tobias.ylander@swipnet.se
  }


  TSystray = class(TComponent)
  private
    { Private declarations }
    FHandle: HWnd;
    FInSysTray: Boolean;
    FEnabled: Boolean;
    FIcon: TIcon;
    FHint: string;
    FPopupMenu: TPopupMenu;
    FNotifyIconData: TNotifyIconData;
    FOnDblClick: TNotifyEvent;
    FOnLeftClick: TNotifyEvent;
    FOnRightClick: TNotifyEvent;
    procedure WndProc(var Message: TMessage);
    procedure CreateIcon;
    procedure UpdateIcon;
    procedure DestroyIcon;
    function GetIconHandle: HIcon;
    procedure SetEnabled(Value: Boolean);
    procedure SetIcon(Value: TIcon);
    procedure SetHint(Value: string);
    procedure SetPopupMenu(Value: TPopupMenu);
    procedure Minimize(Sender: TObject); virtual;
    procedure Restore(Sender: TObject); virtual;
    procedure DefaultOnDblClickEvent(Sender: TObject);
    procedure DefaultOnRightClickEvent(Sender: TObject);
  protected
    { Protected declarations }
    procedure DblClick; virtual;
    procedure LeftClick; virtual;
    procedure RightClick; virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Handle: HWnd read FHandle;
    property InSystray: Boolean read FInSystray;
  published
    { Published declarations }
    // Properties
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Icon: TIcon read FIcon write SetIcon;
    property Hint: string read FHint write SetHint;
    property PopupMenu: TPopupMenu read FPopupMenu write SetPopupMenu default nil;
    // Events
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnLeftClick: TNotifyEvent read FOnLeftClick write FOnLeftClick;
    property OnRightClick: TNotifyEvent read FOnRightClick write FOnRightClick;
  end;

  TForm1 = class(TForm)
    pagecontrol: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    dvdregion: TRadioGroup;
    GroupBox1: TGroupBox;
    Button1: TButton;
    Button2: TButton;
    Button4: TButton;
    Button3: TButton;
    macrovision: TCheckBox;
    opendialog: TOpenDialog;
    station: TEdit;
    Label1: TLabel;
    SpeedButton1: TSpeedButton;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    drivescombo: TComboBox;
    defaultdrive: TCheckBox;
    autoplay: TCheckBox;
    Memo1: TMemo;
    Label3: TLabel;
    version: TLabel;
    Label4: TLabel;
    languages: TComboBox;
    Memo2: TMemo;
    popup: TPopupMenu;
    region: TMenuItem;
    regionnone: TMenuItem;
    region0: TMenuItem;
    region1: TMenuItem;
    region2: TMenuItem;
    region3: TMenuItem;
    region4: TMenuItem;
    region5: TMenuItem;
    region6: TMenuItem;
    Macrovision1: TMenuItem;
    button1p: TMenuItem;
    button2p: TMenuItem;
    button3p: TMenuItem;
    button4p: TMenuItem;
    minimizetotray: TCheckBox;
    startminimized: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure defaultdriveClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure autoplayClick(Sender: TObject);
    procedure languagesChange(Sender: TObject);
    procedure macrovisionClick(Sender: TObject);
    procedure dvdregionClick(Sender: TObject);
    procedure Macrovision1Click(Sender: TObject);
    procedure regionnoneClick(Sender: TObject);
    procedure region0Click(Sender: TObject);
    procedure region1Click(Sender: TObject);
    procedure region2Click(Sender: TObject);
    procedure region3Click(Sender: TObject);
    procedure region4Click(Sender: TObject);
    procedure region5Click(Sender: TObject);
    procedure region6Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure LaunchDVD(Sender: TObject);
    function LoadLanguage(Sender : TObject):boolean;
    procedure LoadLanguagePacks(Sender : TObject);
    function SetValue(codekey:string;value:string):boolean;
    procedure UpdatePopup(itemindex:integer);
  end;

var
  Form1: TForm1;
  sdudf:HMODULE;
  sdudfversion:word;
  udfopen:pointer;
  station_hpid:THandle;
  localarray:array[0..BYTES_TO_DUMP-1] of byte;
  info: TStartupInfo;
  procc:TProcessInformation;
  byteswritten,bytesread:DWORD;
  patch : tPatch;
  shifted_dvd_region,dvd_region: byte;
  setzeroregioncode,setregioncode,setmacrovision:boolean;
  customer:string;
  commandline:string;
  Reg:TRegistry;
  SysTray:TSysTray;
  timer_id:UINT;
const
  zs_cant_update_registry:string = 'Cannot update registry';
  zs_cant_start_station:string = 'Cannot start DVD Station';
  zs_cant_retrieve_sdudf_version:string = 'Cannot retrieve version of ';
  zs_invalid_sdudf_version:string = 'Invalid version found in ';

implementation

{$R *.DFM}

procedure LTrim(var mit:string);
begin
  while (length(mit)>0) and (mit[1]<=#32) do
    delete(mit,1,1);
end;

procedure RTrim(var mit:string);
begin
  while (length(mit)>0) and (mit[length(mit)]<=#32) do
    delete(mit,length(mit),1);
end;

procedure AllTrim(var mit:string);
begin
  Ltrim(mit);
  Rtrim(mit);
end;

function GetFileInfo(const filename: string;var fileinfo:TFileInfo):boolean;
var
  Size: DWORD;
  Dummy: DWORD;
  VerRes: array [0..4095] of byte;
  pTransTab: Pointer;
  strCSet: string;
  Buf: Pointer;

begin
  Size:= GetFileVersionInfoSize(PChar(FileName), Dummy);
  if Size > sizeof(VerRes) then
   begin
    result:=False;
    exit;
   end;

  if (not GetFileVersionInfo(PChar(FileName), 0, Size, Addr(VerRes))) then
    begin
     result:=False;
     exit;
    end;

  if (not VerQueryValue(Addr(VerRes), '\\VarFileInfo\\Translation', pTransTab, Size)) then
    begin
     result:=False;
     exit;
    end;

  strCSet:= '\\StringFileInfo\\' +
    IntToHex(LoWord(Longint(pTransTab^)), 4) +
    IntToHex(HiWord(Longint(pTransTab^)), 4) + '\\';

  { Preset }
  fileinfo.Comments:= '';
  fileinfo.CompanyName:= '';
  fileinfo.FileDescription:= '';
  fileinfo.FileVersion:= '';
  fileinfo.InternalName:= '';
  fileinfo.LegalCopyright:= '';
  fileinfo.LegalTrademarks:= '';
  fileinfo.OriginalFilename:= '';
  fileinfo.PrivateBuild:= '';
  fileinfo.ProductName:= '';
  fileinfo.ProductVersion:= '';
  fileinfo.SpecialBuild:= '';

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'Comments'), Buf, Size) then
   fileinfo.Comments:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'CompanyName'), Buf, Size) then
   fileinfo.CompanyName:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'FileDescription'), Buf, Size) then
   fileinfo.FileDescription:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'FileVersion'), Buf, Size) then
   fileinfo.FileVersion:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'InternalName'), Buf, Size) then
   fileinfo.InternalName:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'LegalCopyright'), Buf, Size) then
   fileinfo.LegalCopyright:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'LegalTrademarks'), Buf,  Size) then
   fileinfo.LegalTrademarks:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'OriginalFilename'), Buf, Size) then
   fileinfo.OriginalFilename:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'PrivateBuild'), Buf, Size) then
   fileinfo.PrivateBuild:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'ProductName'), Buf, Size) then
   fileinfo.ProductName:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'ProductVersion'), Buf, Size) then
   fileinfo.ProductVersion:= PChar(Buf);

  if VerQueryValue(Addr(VerRes), PChar(strCSet + 'SpecialBuild'), Buf, Size) then
   fileinfo.SpecialBuild:= PChar(Buf);

  Result:=True;
end;


function LoadFromReg(rootkey:HKEY;base:string;ident:string;value:pointer):boolean;
var
  regtype:TRegDataType;
begin
  reg.rootkey := rootkey;

  if not reg.Openkey('SOFTWARE\'+base,FALSE) then
   begin
    result:=False;
    exit;
   end;

  if not reg.ValueExists(Ident) then
    begin
     reg.CloseKey;
     result:=false;
     exit;
    end;

  regtype:=reg.GetDataType(ident);

  if (regtype=rdUnknown) then
   begin
    reg.CloseKey;
    Result:=False;
    exit;
   end;

  try
    if (regtype=rdString) or (regtype=rdExpandString) then
      PString(value)^:=reg.ReadString(ident)
    else if (regtype=rdInteger) then
      PInteger(value)^:=reg.ReadInteger(ident);
  except
    reg.CloseKey;
    result:=false;
    exit;
  end;

  reg.CloseKey;
  Result:=True;
end;

function WriteToReg(rootkey:HKEY;base:string;ident:string;value:pointer;regtype:TRegDataType;cancreate:boolean):boolean;
begin
  reg.rootkey := rootkey;

  if not reg.OpenKey('SOFTWARE\'+base,cancreate) then
   begin
    result:=False;
    exit;
   end;

  try
    if (regtype=rdString) or (regtype=rdExpandString) then
      reg.WriteString(ident,PString(value)^)
    else if (regtype=rdInteger) then
      reg.WriteInteger(ident,PInteger(value)^);
  except
    MessageDlg(zs_cant_update_registry, mtInformation, [mbOk], 0);
    reg.CloseKey;
    result:=False;
    exit;
  end;

  reg.CloseKey;
  Result:=True;
end;

procedure LoadPatch(var patch:tPatch;sdudfversion:integer);
var
  i:integer;
begin
  patch.num_of_patches := 8;

  for i:=0 to MAX_PATCHENTRIES-1 do
   begin
    with patch.patch[i] do
     begin
      location:=-1;
      applicable:=false;
      version:=0;
     end;
   end;

  with patch.patch[0] do
    begin
      switch_on := @setregioncode;
      version:=SKIP_VERSION_CHECK;
      applicable:=sdudfversion>=LOWEST_SUPPORTED_VERSION;

      num_of_patchbytes := 6;
      num_of_params := 1;

      src[0]:= $3a;
      src[1]:= $c3;
      src[2]:= $76;
      src[3]:= $04;
      src[4]:= $3c;
      src[5]:= $07;

      dst[0]:= $b0;
      dst[1]:= $01;
      dst[2]:= $eb;
      dst[3]:= $08;
      dst[4]:= $3c;
      dst[5]:= $07;

      param[0].index := 1;
      param[0].paramptr := @dvd_region;
    end;

  with patch.patch[1] do
    begin
      switch_on := @setregioncode;
      version:=SKIP_VERSION_CHECK;
      applicable:=sdudfversion>=LOWEST_SUPPORTED_VERSION;

      num_of_patchbytes := 7;
      num_of_params := 1;

      src[0]:= $fe;
      src[1]:= $c1;
      src[2]:= $b8;
      src[3]:= $01;
      src[4]:= $00;
      src[5]:= $00;
      src[6]:= $00;

      dst[0]:= $b1;
      dst[1]:= $01;
      dst[2]:= $b8;
      dst[3]:= $01;
      dst[4]:= $00;
      dst[5]:= $00;
      dst[6]:= $00;

      param[0].index := 1;
      param[0].paramptr := @dvd_region;
    end;

  with patch.patch[2] do
    begin
      switch_on := @setzeroregioncode;
      version:=125;
      applicable:=sdudfversion=125;

      num_of_patchbytes := 9;
      num_of_params := 1;

      src[0]:= $8a;
      src[1]:= $54;
      src[2]:= $0c;
      src[3]:= $09;
      src[4]:= $88;
      src[5]:= $10;
      src[6]:= $b8;
      src[7]:= $01;
      src[8]:= $00;

      dst[0]:= $b2;
      dst[1]:= $00;
      dst[2]:= $90;
      dst[3]:= $90;
      dst[4]:= $88;
      dst[5]:= $10;
      dst[6]:= $b8;
      dst[7]:= $01;
      dst[8]:= $00;

      param[0].index := 1;
      param[0].paramptr := @shifted_dvd_region;
    end;

  with patch.patch[3] do
    begin
      switch_on := @setzeroregioncode;
      version:=137;
      applicable:=sdudfversion=137;

      num_of_patchbytes := 9;
      num_of_params := 1;

      src[0]:= $8a;
      src[1]:= $54;
      src[2]:= $0c;
      src[3]:= $0d;
      src[4]:= $88;
      src[5]:= $10;
      src[6]:= $b8;
      src[7]:= $01;
      src[8]:= $00;

      dst[0]:= $b2;
      dst[1]:= $00;
      dst[2]:= $90;
      dst[3]:= $90;
      dst[4]:= $88;
      dst[5]:= $10;
      dst[6]:= $b8;
      dst[7]:= $01;
      dst[8]:= $00;

      param[0].index := 1;
      param[0].paramptr := @shifted_dvd_region;
    end;

  with patch.patch[4] do
    begin
      switch_on := @setzeroregioncode;
      version:=139;
      applicable:=sdudfversion=139;

      num_of_patchbytes := 9;
      num_of_params := 1;

      src[0]:= $8a;
      src[1]:= $54;
      src[2]:= $0c;
      src[3]:= $0d;
      src[4]:= $88;
      src[5]:= $10;
      src[6]:= $b8;
      src[7]:= $01;
      src[8]:= $00;

      dst[0]:= $b2;
      dst[1]:= $00;
      dst[2]:= $90;
      dst[3]:= $90;
      dst[4]:= $88;
      dst[5]:= $10;
      dst[6]:= $b8;
      dst[7]:= $01;
      dst[8]:= $00;

      param[0].index := 1;
      param[0].paramptr := @shifted_dvd_region;
    end;

  with patch.patch[5] do
    begin
      switch_on := @setmacrovision;
      version:=125;
      applicable:=sdudfversion=125;

      num_of_patchbytes := 7;
      num_of_params := 0;

      src[0]:= $8a;
      src[1]:= $44;
      src[2]:= $24;
      src[3]:= $0f;
      src[4]:= $5f;
      src[5]:= $88;
      src[6]:= $03;

      dst[0]:= $30;
      dst[1]:= $c0;
      dst[2]:= $90;
      dst[3]:= $90;
      dst[4]:= $5f;
      dst[5]:= $88;
      dst[6]:= $03;
    end;

  with patch.patch[6] do
    begin
      switch_on := @setmacrovision;
      version:=137;
      applicable:=sdudfversion=137;

      num_of_patchbytes := 10;
      num_of_params := 0;

      src[0]:= $8a;
      src[1]:= $54;
      src[2]:= $24;
      src[3]:= $0b;
      src[4]:= $33;
      src[5]:= $c0;
      src[6]:= $88;
      src[7]:= $17;
      src[8]:= $5f;
      src[9]:= $5e;

      dst[0]:= $30;
      dst[1]:= $d2;
      dst[2]:= $90;
      dst[3]:= $90;
      dst[4]:= $33;
      dst[5]:= $c0;
      dst[6]:= $88;
      dst[7]:= $17;
      dst[8]:= $5f;
      dst[9]:= $5e;
    end;

  with patch.patch[7] do
    begin
      switch_on := @setmacrovision;
      version:=139;
      applicable:=sdudfversion=139;

      num_of_patchbytes := 8;
      num_of_params := 0;

      src[0]:= $8a;
      src[1]:= $54;
      src[2]:= $24;
      src[3]:= $0b;
      src[4]:= $88;
      src[5]:= $17;
      src[6]:= $33;
      src[7]:= $c0;

      dst[0]:= $30;
      dst[1]:= $d2;
      dst[2]:= $90;
      dst[3]:= $90;
      dst[4]:= $88;
      dst[5]:= $17;
      dst[6]:= $33;
      dst[7]:= $c0;
    end;
end;

function patchaddress(var sourcestream:array of byte;sourcestreamlen:longint;var patchentry:tPatchEntry):boolean;
var
 i,j:longint;
 writtenmemory:boolean;
begin
 for i:=0 to sourcestreamlen-1 do
   begin
    if comparemem(Addr(sourcestream[i]),Addr(patchentry.src),patchentry.num_of_patchbytes) then
      begin
       patchentry.location:=i;

       for j:=0 to patchentry.num_of_params-1 do
         patchentry.dst[patchentry.param[j].index]:=patchentry.param[j].paramptr^;

       writtenmemory:=WriteProcessMemory(station_hpid,
                                         pointer(integer(udfopen)+patchentry.location),
                                         Addr(patchentry.dst),
                                         patchentry.num_of_patchbytes,
                                         byteswritten);

       if writtenmemory and (byteswritten=patchentry.num_of_patchbytes) then
        begin
         Result:=True;
         exit;
        end;
      end;
   end;

 Result:=False;
end;

procedure PatchProcess(Wnd: HWnd; Msg, TimerID, SysTime: Longint); stdcall;
var
 i:integer;
begin
 if (station_hpid=0) then
   exit;

 if (udfopen=NIL) then
   begin
    sdudf := LoadLibrary(SDUDFFILE);

    if (sdudf<=HINSTANCE_ERROR) then
      exit;

    udfopen := GetProcAddress(sdudf,'UDFOpen');
    FreeLibrary(sdudf);
    exit;
   end;

 if (udfopen=NIL) then
   exit;

 if not ReadProcessMemory(station_hpid,
                          udfopen,
                          addr(localarray),
                          sizeof(localarray),
                          bytesread) then
   exit;

 {Let's patch}
 for i:=0 to patch.num_of_patches-1 do
  begin
   if patch.patch[i].applicable and patch.patch[i].switch_on^ then
    PatchAddress(localarray,bytesread,patch.patch[i]);
  end;
end;

function CheckForDVD(dvdDrive:char):boolean;
const
 dvdfile : string = 'z:\video_ts\Video_ts.ifo';
var
 rfile:integer;
begin
 dvdfile[1]:=dvdDrive;
 rfile:=fileopen(dvdfile,fmOpenRead);
 if (rfile<>-1) then
   fileclose(rfile);

 Result:=rfile<>-1;
end;

procedure TForm1.LaunchDVD;
var
 dvd_drive:string;
begin
 dvd_drive:=drivescombo.items.strings[drivescombo.itemindex];
 if (length(dvd_drive)>0) and (CheckForDVD(dvd_drive[1])) then
   Button1Click(Sender);
end;

procedure TForm1.Button1Click(Sender: TObject);
var
 drive:integer;
 drivestr:string;
begin
 fillchar(localarray,sizeof(localarray),0);
 fillchar(info,sizeof(info),0);
 info.cb:=sizeof(info);

 if (defaultdrive.state=cbUnChecked) then
   begin
    drivestr:=drivescombo.items.strings[drivescombo.itemindex];
    drivestr:=UpperCase(drivestr);
    drive:=byte(drivestr[1])-byte('A');
    WriteToReg(HKEY_LOCAL_MACHINE,'Sigma Designs\REALmagic\Station','Drive',addr(drive),rdInteger,FALSE);
   end;

 if not CreateProcess(PChar(station.text),PChar(commandline),NIL,NIL,
                      FALSE,NORMAL_PRIORITY_CLASS,
                      NIL,NIL,info,procc) then
  begin
   MessageDlg(zs_cant_start_station, mtInformation, [mbOk], 0);
   exit;
  end;

 station_hpid:=procc.hProcess;

 if (timer_id=0) then
  begin
    timer_id := SetTimer(0,0,100,Addr(PatchProcess));
    if (timer_id = 0) then
      exit;
  end;

 button1.enabled:=False;
 button2.enabled:=False;
 button1p.enabled:=False;
 button2p.enabled:=False;
end;

procedure LongStringToTString(dest:Tstrings;mit:string);
var
 i:integer;
begin
 dest.Clear;
 i:=pos(char(255),mit);
 while (i>0) do
  begin
    dest.Add(copy(mit,1,i-1));
    delete(mit,1,i);
    i:=pos(char(255),mit);
  end;
 dest.Add(mit);
end;

function TForm1.SetValue;
var
 ret:boolean;
begin
 ret:=True;
 AllTrim(codekey);
 codekey:=LowerCase(codekey);
 if (codekey='zs_title') then
  Form1.Caption := Value
 else if (codekey='zs_launch_sheet') then
  Tabsheet1.caption:=Value
 else if (codekey='zs_region_title') then
  dvdregion.caption:=Value
 else if (codekey='zs_region_none') then
  begin
   dvdregion.items.Strings[0]:=Value;
   regionnone.caption:=Value;
  end
 else if (codekey='zs_region_0') then
  begin
   dvdregion.items.Strings[1]:=Value;
   region0.caption:=Value;
  end
 else if (codekey='zs_region_1') then
  begin
   dvdregion.items.Strings[2]:=Value;
   region1.caption:=Value;
  end
 else if (codekey='zs_region_2') then
  begin
   dvdregion.items.Strings[3]:=Value;
   region2.caption:=Value;
  end
 else if (codekey='zs_region_3') then
  begin
   dvdregion.items.Strings[4]:=Value;
   region3.caption:=Value;
  end
 else if (codekey='zs_region_4') then
  begin
   dvdregion.items.Strings[5]:=Value;
   region4.caption:=Value;
  end
 else if (codekey='zs_region_5') then
  begin
   dvdregion.items.Strings[6]:=Value;
   region5.caption:=Value;
  end
 else if (codekey='zs_region_6') then
  begin
   dvdregion.items.Strings[7]:=Value;
   region6.caption:=Value;
  end
 else if (codekey='zs_run_station_title') then
  groupbox1.caption:=Value
 else if (codekey='zs_run_dvd_btn') then
  begin
   button1.caption:=Value;
   button1p.caption:=Value;
  end
 else if (codekey='zs_run_play_file_btn') then
  begin
   button2.caption:=Value;
   button2p.caption:=Value;
  end
 else if (codekey='zs_quit_btn') then
  begin
   button4.caption:=Value;
   button4p.caption:=Value;
  end
 else if (codekey='zs_save_btn') then
  begin
   button3.caption:=Value;
   button3p.caption:=Value;
  end
 else if (codekey='zs_configuration_sheet') then
  Tabsheet2.caption:=Value
 else if (codekey='zs_locate_station_txt') then
  Label1.caption:=Value
 else if (codekey='zs_driver_version_txt') then
  Label3.caption:=Value
 else if (codekey='zs_dvd_options') then
  groupbox2.caption:=Value
 else if (codekey='zs_dvd_drive') then
  Label2.caption:=Value
 else if (codekey='zs_default_drive') then
  defaultdrive.caption:=Value
 else if (codekey='zs_autoplay_dvd') then
  autoplay.caption:=Value
 else if (codekey='zs_macrovision_sheet') then
  Tabsheet3.caption:=Value
 else if (codekey='zs_disable_macrovision') then
  begin
   macrovision.caption:=Value;
   macrovision1.caption:=Value;
  end
 else if (codekey='zs_macrovision_disclaimer_txt') then
  LongStringToTString(memo1.Lines,value)
 else if (codekey='zs_help_sheet') then
  Tabsheet4.caption:=Value
 else if (codekey='zs_help_txt') then
  LongStringToTString(Memo2.Lines,value)
 else if (codekey='zs_cant_update_registry') then
   zs_cant_update_registry:=Value
 else if (codekey='zs_cant_start_station') then
   zs_cant_start_station:=Value
 else if (codekey='zs_cant_retrieve_sdudf_version') then
   zs_cant_retrieve_sdudf_version:=Value
 else if (codekey='zs_invalid_sdudf_version') then
   zs_invalid_sdudf_version:=Value
 else if (codekey='zs_language') then
   Label4.Caption:=Value
 else if (codekey='zs_start_minimized') then
   startminimized.caption:=Value
 else if (codekey='zs_minimize_to_tray') then
   minimizetotray.caption:=Value
 else if (codekey='zs_dvd_region') then
   region.caption:=Value
 else
   ret:=False;

 Result:=ret;
end;

function TForm1.LoadLanguage;
var
 rfile:textfile;
 line,nextline:string;
 i:integer;
 codekey:string;
 value:string;
begin
 assignfile(rfile,'lng'+customer+'pak.txt');
 {$I-} reset(rfile); {$I+}
 if ioresult<>0 then
   begin
     MessageDlg('Cannot open '+'lng'+customer+'pak.txt'+'.',mtInformation, [mbOk], 0);
     Result:=False;
     exit;
   end;

 while not eof(rfile) do
  begin
    readln(rfile,line);
    LTrim(line);

    while (line[1]=COMMENT_CHAR) and not eof(rfile) do
     begin
       readln(rfile,line);
       LTrim(line);
     end;

    i := pos('=',line);
    if (i<>0) then
      begin
        codekey := copy(line,1,i-1);
        AllTrim(codekey);
        delete(line,1,i);
        AllTrim(line);
        while (length(line)>0) and (line[length(line)]=STRING_SEP_CHAR) and not eof(rfile) do
          begin
            AllTrim(line);
            while (line[length(line)]<>char(39)) and (Length(line)>0) do
              delete(line,length(line),1);
            if (length(line)>0) then
              delete(line,length(line),1);
            readln(rfile,nextline);
            AllTrim(nextline);
            delete(nextline,1,1);
            line:=line+char(255)+nextline;
          end;
        delete(line,1,1);
        Rtrim(line);
        i:=length(line);
        if (i>0) and (line[i]=#39) then
          delete(line,i,1);
        value:=Line;

        if not SetValue(codekey,value) then
         begin
          closefile(rfile);
          MessageDlg('Invalid tag ('+codekey+') in language pack ('+customer+').',mtInformation, [mbOk], 0);
          result:=False;
          exit;
         end;
      end;
  end;
 closefile(rfile);
 Result:=True;
end;

procedure TForm1.LoadLanguagePacks;
var
 rfile:textfile;
 rek:TSearchRec;
 line:string;
 found:integer;
 i:Integer;
begin
 languages.items.Clear;

 found:=findfirst('.\lng??pak.txt',faAnyFile,rek);
 while (found=0) do
  begin
   assignfile(rfile,rek.name);
   {$I-} reset(rfile); {$I+}
   if ioresult=0 then
    begin
      while not eof(rfile) do
       begin
         readln(rfile,line);
         AllTrim(line);
         while (line[1]<>COMMENT_CHAR) and not eof(rfile) do
          begin
            readln(rfile,line);
            AllTrim(line);
          end;
         i:=pos(LNGPAK_ID,line);
         if (i>0) then
          begin
            delete(line,1,i+length(LNGPAK_ID));
            AllTrim(line);
            delete(line,1,pos('=',line)+1);
            AllTrim(line);

            languages.items.Add(line);
          end;
       end;
      closefile(rfile);
    end;
    found:=findnext(rek);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i,auto_play,ismacro,istotray,isstartminimized:integer;
  default_drive,zone:integer;
  set_customer,stationexe:string;
  dvd_drive:string;
  driveToTest:string;
  dllinfo:TFileInfo;
  dllinfostring:string;
begin
  Reg:=TRegistry.Create;

  udfopen:=NIL;
  station_hpid:=0;
  timer_id:=0;

  commandline:='';
  setregioncode:=False;
  setmacrovision:=False;
  macrovision.state:=cbUnChecked;
  station.text:='c:\Realmagic\station.exe';
  autoplay.state:=cbChecked;
  defaultdrive.state:=cbChecked;
  dvdregion.itemindex:=0;

  customer:='US';

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','language',addr(set_customer)) then
    begin
      set_customer := UpperCase(set_customer);
      AllTrim(set_customer);
      customer := copy(set_customer,1,2);
    end;

  if not LoadLanguage(Sender) then
    customer:='US'; {reverting to US}

  LoadLanguagePacks(Sender);

  languages.enabled:=languages.items.count >= 1;
  for i:=0 to languages.items.count-1 do
   if pos('('+customer+')',languages.Items.Strings[i])>0 then
     begin
      languages.ItemIndex:=i;
      break;
     end;

  if (not GetFileInfo(SDUDFFILE,dllinfo)) then
   begin
    MessageDlg(zs_cant_retrieve_sdudf_version+SDUDFFILE+'.',mtInformation, [mbOk], 0);
    Application.Terminate;
    exit;
   end;

  dllinfostring:=dllinfo.FileVersion;
  AllTrim(dllinfostring);
  delete(dllinfostring,1,Pos('.',dllinfostring));
  delete(dllinfostring,1,Pos('.',dllinfostring));
  delete(dllinfostring,Pos('(',dllinfostring),length(dllinfostring));
  AllTrim(dllinfostring);

  try
    sdudfversion:=strtoint(dllinfostring);
  except
    MessageDlg(zs_invalid_sdudf_version+SDUDFFILE+'.',mtInformation, [mbOk], 0);
    Application.Terminate;
    exit;
  end;

  version.caption:='Build '+inttostr(sdudfversion);

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','minimizetotray',addr(istotray)) then
    begin
      if (istotray=1) then
       begin
        minimizetotray.state:=cbChecked;
        Systray:=TSysTray.Create(Application);
        Systray.PopupMenu:=Popup;
       end
      else
        minimizetotray.state:=cbUnChecked;
    end;

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','startminimized',addr(isstartminimized)) then
    begin
      if (isstartminimized=1) then
       begin
        startminimized.state:=cbChecked;
        WindowState:=wsMinimized;
       end
      else
        startminimized.state:=cbUnChecked;
    end;

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','macrovision',addr(ismacro)) then
    begin
      if (ismacro=1) then
        macrovision.state:=cbChecked
      else
        macrovision.state:=cbUnChecked;
    end;

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','zone',addr(zone)) then
    begin
      if (zone<0) or (zone>7) then
        zone:=0;
      dvdregion.itemindex:=zone;
    end;

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','station_exe',addr(stationexe)) then
    begin
      station.text:=stationexe;
    end;

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','auto_play',addr(auto_play)) then
    begin
      if (auto_play=1) then
        autoplay.state:=cbChecked
      else
        autoplay.state:=cbUnChecked;
    end;

  driveToTest:='A:\';
  while (driveToTest[1]<>'Z') do
   begin
    if (GetDriveType(PChar(driveToTest)) = DRIVE_CDROM) then
      DrivesCombo.Items.Add(driveToTest);
    driveToTest[1]:=char(byte(driveToTest[1])-1);
   end;

  DrivesCombo.ItemIndex:=0;

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','dvd_drive',addr(dvd_drive)) then
    begin
      for i:=0 to drivescombo.items.count-1 do
        if pos(dvd_drive,DrivesCombo.Items.Strings[i])>0 then
          begin
           DrivesCombo.ItemIndex:=i;
           break;
          end;
    end;

  if LoadFromReg(DEFAULT_REGBASE,'Zone Selector','default_drive',addr(default_drive)) then
    begin
      if (default_drive=1) then
        defaultdrive.state:=cbChecked
      else
        defaultdrive.state:=cbUnChecked;
    end;

  drivescombo.enabled:=defaultdrive.state=cbUnChecked;

  UpdatePopup(dvdregion.itemindex);

  LoadPatch(patch,sdudfversion);

  if (autoplay.state=cbChecked) then
    LaunchDVD(Sender);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 if (timer_id<>0) then
   KillTimer(0,timer_id);

 closehandle(procc.hProcess);
 closehandle(procc.hThread);
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
 Application.Terminate;
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
 opendialog.filename:=ExtractFileName(station.text);
 opendialog.initialdir:=ExtractFilePath(station.text);

 if opendialog.execute then
   station.text:=opendialog.filename;
end;


procedure TForm1.Button3Click(Sender: TObject);
var
  default_drive,auto_play,ismacro,istotray,isstartminimized:integer;
  zone:integer;
  dvd_drive,stationexe:string;
begin
  ismacro:=integer(macrovision.state=cbChecked);
  istotray:=integer(minimizetotray.state=cbChecked);
  isstartminimized:=integer(startminimized.state=cbChecked);

  default_drive:=integer(defaultdrive.state=cbChecked);
  zone:=dvdregion.itemindex;
  stationexe:=station.text;
  auto_play:=integer(autoplay.state=cbChecked);
  dvd_drive:=drivescombo.items.strings[drivescombo.itemindex];

  WriteToReg(DEFAULT_REGBASE,'Zone Selector','macrovision',addr(ismacro),rdInteger,TRUE);
  WriteToReg(DEFAULT_REGBASE,'Zone Selector','zone',addr(zone),rdInteger,TRUE);
  WriteToReg(DEFAULT_REGBASE,'Zone Selector','station_exe',addr(stationexe),rdString,TRUE);
  WriteToReg(DEFAULT_REGBASE,'Zone Selector','auto_play',addr(auto_play),rdInteger,TRUE);
  WriteToReg(DEFAULT_REGBASE,'Zone Selector','dvd_drive',addr(dvd_drive),rdString,TRUE);
  WriteToReg(DEFAULT_REGBASE,'Zone Selector','default_drive',addr(default_drive),rdInteger,TRUE);
  WriteToReg(DEFAULT_REGBASE,'Zone Selector','language',addr(customer),rdString,TRUE);
  WriteToReg(DEFAULT_REGBASE,'Zone Selector','startminimized',addr(isstartminimized),rdInteger,TRUE);
  WriteToReg(DEFAULT_REGBASE,'Zone Selector','minimizetotray',addr(istotray),rdInteger,TRUE);
end;

procedure TForm1.defaultdriveClick(Sender: TObject);
begin
  drivescombo.enabled:=defaultdrive.state=cbUnChecked;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
 opendialog.filename:='';
 opendialog.initialdir:=drivescombo.items.strings[drivescombo.itemindex];

 if not opendialog.execute then
  begin
   commandline:='';
   exit;
  end;

 commandline:='/f '+opendialog.filename;
 Button1Click(Sender);
end;

procedure TForm1.autoplayClick(Sender: TObject);
begin
  if (autoplay.state=cbChecked) then
    LaunchDVD(Sender);
end;


procedure TForm1.languagesChange(Sender: TObject);
var
 save_customer,new_customer:string;
 i:Integer;
begin
 new_customer:=languages.items.strings[languages.itemindex];
 i:=pos('(',new_customer);
 if (i>0) then
  begin
   delete(new_customer,1,i);
   i:=pos(')',new_customer);
   delete(new_customer,i,length(new_customer));
   new_customer:=UpperCase(new_customer);
   AllTrim(new_customer);
   save_customer:=customer;
   customer:=new_customer;
   if not LoadLanguage(Sender) then
     begin
       customer:=save_customer;
       LoadLanguage(Sender);
     end;
  end;
end;

procedure TForm1.macrovisionClick(Sender: TObject);
begin
 setmacrovision := macrovision.state=cbChecked;
 macrovision1.checked:=setmacrovision;
end;

procedure TForm1.dvdregionClick(Sender: TObject);
begin
 setregioncode := dvdregion.itemindex<>0;
 setzeroregioncode:=False;

 if setregioncode then
  begin
   dvd_region:=dvdregion.itemindex-1;
   if (dvd_region>0) then
     shifted_dvd_region:= (-1) shl dvd_region
   else
     shifted_dvd_region:=0;

   if (dvd_region=0) then
    begin
      setzeroregioncode:=True;
      setregioncode:=false;
    end;
  end;
 UpdatePopup(dvdregion.itemindex);
end;

{ TSystray }

procedure TSystray.WndProc(var Message: TMessage);
begin
  with Message do
    if Msg = WM_SYSTRAY then
    begin
      if FEnabled then
        case LParam of
          WM_LBUTTONDOWN: LeftClick;
          WM_LBUTTONDBLCLK: DblClick;
          WM_RBUTTONDOWN: RightClick;
        end;
    end
    else
      Result := DefWindowProc(FHandle, Msg, wParam, lParam);
end;

procedure TSystray.CreateIcon;
begin
  if not FInSystray and not (csDesigning in ComponentState) then
  begin
    FillChar(FNotifyIconData, SizeOf(TNotifyIconData), 0);
    with FNotifyIconData do
    begin
      cbSize := SizeOf(TNotifyIconData);
      uID := 1;
      Wnd := FHandle;

      uFlags := uFlags or NIF_MESSAGE or NIF_ICON;
      if Length(FHint) > 0 then
      begin
        uFlags := uFlags or NIF_TIP;
        StrPCopy(szTip, PChar(FHint));
      end;

      uCallbackMessage := WM_SYSTRAY;
      hIcon := GetIconHandle;
    end;
    Shell_NotifyIcon(NIM_ADD, @FNotifyIconData);
    FInSystray := True;
  end;
end;

procedure TSystray.UpdateIcon;
begin
  if FInSystray and not (csDesigning in ComponentState) then
  begin
    with FNotifyIconData do
    begin
      if Length(FHint) > 0 then
      begin
        if uFlags and NIF_TIP = 0 then uFlags := uFlags or NIF_TIP;
        StrPCopy(szTip, PChar(FHint));
      end
      else
        if uFlags and NIF_TIP <> 0 then uFlags := uFlags and not NIF_TIP;

      hIcon := GetIconHandle;
      Shell_NotifyIcon(NIM_MODIFY, @FNotifyIconData);
    end;
  end;
end;

procedure TSystray.DestroyIcon;
begin
  if FInSystray and not (csDesigning in ComponentState) then
  begin
    Shell_NotifyIcon(NIM_DELETE, @FNotifyIconData);
    FInSystray := False;
  end;
end;

function TSystray.GetIconHandle: HIcon;
begin
  Result := FIcon.Handle;
  if Result = 0 then Result := Application.Icon.Handle;
end;

procedure TSystray.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    if Application.MainForm.WindowState = wsMinimized then
    begin
      case FEnabled of
        True:
          begin
            CreateIcon;
            ShowWindow(Application.Handle, SW_HIDE);
          end;
        False:
          begin
            DestroyIcon;
            ShowWindow(Application.Handle, SW_SHOW);
          end;
      end;
    end;
  end;
end;

procedure TSystray.SetIcon(Value: TIcon);
begin
  if Value <> FIcon then
  begin
    FIcon.Assign(Value);
    if FEnabled and FInSystray then UpdateIcon;
  end;
end;

procedure TSystray.SetHint(Value: string);
begin
  if Value <> FHint then
  begin
    FHint := Value;
    if FEnabled and FInSystray then UpdateIcon;
  end;
end;

procedure TSystray.SetPopupMenu(Value: TPopupMenu);
begin
  if Value <> FPopupMenu then
  begin
    FPopupMenu := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TSystray.Minimize(Sender: TObject);
begin
  if FEnabled and not FInSystray then
  begin
    CreateIcon;
    ShowWindow(Application.Handle, SW_HIDE);
  end;
end;

procedure TSystray.Restore(Sender: TObject);
begin
  if FEnabled and FInSystray then
  begin
    SetForegroundWindow(Application.Handle);
    DestroyIcon;
    ShowWindow(Application.Handle, SW_SHOW);
  end;
end;

procedure TSystray.DefaultOnDblClickEvent(Sender: TObject);
begin
  Application.Restore;
end;

procedure TSystray.DefaultOnRightClickEvent(Sender: TObject);
var
  P: TPoint;
begin
  if Assigned(FPopupMenu) then
  begin
    GetCursorPos(P);
    FPopupMenu.PopupComponent := Self;
    FPopupMenu.Popup(P.X, P.Y);
    PostMessage(Application.Handle, WM_NULL, 0, 0);
  end;
end;

procedure TSystray.DblClick;
begin
  if Assigned(FOnDblClick) then FOnDblClick(Self);
end;

procedure TSystray.LeftClick;
begin
  if Assigned(FOnLeftClick) then FOnLeftClick(Self);
end;

procedure TSystray.RightClick;
begin
  if Assigned(FOnRightClick) then FOnRightClick(Self);
end;

constructor TSystray.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FEnabled := True;
  FInSystray := False;
  FIcon := TIcon.Create;
  FHint := '';
  FPopupMenu := nil;
  FillChar(FNotifyIconData, SizeOf(TNotifyIconData), 0);

  FOnDblClick := DefaultOnDblClickEvent;
  FOnRightClick := DefaultOnRightClickEvent;

  FHandle := AllocateHWnd(WndProc);

  if not (csDesigning in ComponentState) then
  begin
    Application.OnMinimize := Minimize;
    Application.OnRestore := Restore;
  end;
end;

destructor TSystray.Destroy;
begin
  if FHandle <> 0 then DeallocateHWnd(FHandle);
  if FInSystray then DestroyIcon;
  FIcon.Free;
  inherited Destroy;
end;

procedure TForm1.Macrovision1Click(Sender: TObject);
begin
 setmacrovision := not setmacrovision;

 if setmacrovision then
   macrovision.state:=cbChecked
 else
   macrovision.state:=cbUnChecked;

 macrovision1.checked:=setmacrovision;
end;

procedure TForm1.UpdatePopup(itemindex:integer);
begin
 regionnone.checked:=itemindex=0;
 region0.checked:=itemindex=1;
 region1.checked:=itemindex=2;
 region2.checked:=itemindex=3;
 region3.checked:=itemindex=4;
 region4.checked:=itemindex=5;
 region5.checked:=itemindex=6;
 region6.checked:=itemindex=7;
end;

procedure TForm1.regionnoneClick(Sender: TObject);
begin
 regionnone.checked:=not regionnone.checked;
 if regionnone.checked then
  begin
   UpdatePopup(0);
   dvdregion.itemindex:=0;
  end;
end;

procedure TForm1.region0Click(Sender: TObject);
begin
 region0.checked:=not region0.checked;
 if region0.checked then
  begin
   UpdatePopup(1);
   dvdregion.itemindex:=1;
  end;
end;

procedure TForm1.region1Click(Sender: TObject);
begin
 region1.checked:=not region1.checked;
 if region1.checked then
  begin
   UpdatePopup(2);
   dvdregion.itemindex:=2;   
  end;
end;

procedure TForm1.region2Click(Sender: TObject);
begin
 region2.checked:=not region2.checked;
 if region2.checked then
  begin
   UpdatePopup(3);
   dvdregion.itemindex:=3;
  end;
end;

procedure TForm1.region3Click(Sender: TObject);
begin
 region3.checked:=not region3.checked;
 if region3.checked then
  begin
   UpdatePopup(4);
   dvdregion.itemindex:=4;
  end;
end;

procedure TForm1.region4Click(Sender: TObject);
begin
 region4.checked:=not region4.checked;
 if region4.checked then
  begin
   UpdatePopup(5);
   dvdregion.itemindex:=5;
  end;
end;

procedure TForm1.region5Click(Sender: TObject);
begin
 region5.checked:=not region5.checked;
 if region5.checked then
  begin
   UpdatePopup(6);
   dvdregion.itemindex:=6;
  end;
end;

procedure TForm1.region6Click(Sender: TObject);
begin
 region6.checked:=not region6.checked;
 if region6.checked then
  begin
   UpdatePopup(7);
   dvdregion.itemindex:=7;
  end;
end;

end.
